;--------------------------- D1_I2.ASM -----------------------------
;
; This program demonstrates duplex (bi-directional) communication
; through the CAN bus.
;
; This program is written to be run in a setup where two CAN units,
; each connected to a computer, communicate with each other via the 
; CAN bus. Keystrokes entered in one computer appear on the screen 
; of the other computer.  
;
; In this program, the transmission of data is initiated by the 
; reception of data through the serial port rather than remote 
; requests from other CAN devices or external signals.
;
; The CAN unit is connected to the host device via a serial channel
; which it monitors constantly. As soon as serial data arrives
; (for example if the user presses a key on the keyboard of the host 
; computer) the program echoes the data back (which appears on the 
; monitor of the host computer) and also transmits it as an ascii
; byte in a CAN package to the CAN bus. 
;
; The CAN unit also monitors the CAN channel continuously. The 
; arrival of a CAN message generates an interrupt upon which the
; program jumps to an interrupt service routine. In the interrupt
; service routine the data segment of the arriving message is 
; extracted (one byte) and sent to the host device via the serial 
; port. 
;
; In short, the incoming CAN messages are handled in an interrupt
; driven fashion while polling is used to monitor the serial channel.
;-------------------------------------------------------------------




; constant declarations
XPOINT       equ  0100h              ; interrupt vector
XPOIC        equ 0F186h              ; interrupt control register
CTL_ST       equ 0EF00h              ; control/status register
INT_ID       equ 0EF02h              ; interrupt register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15



   org XPOINT
   jmpa cc_UC, msg_isr               ; jump to the interrupt service 
                                     ; routine   

   org 04000h

   calla cc_UC, initialize_port
   calla cc_UC, initialize_can       ; initialize CAN registers  

listen_port:
   calla cc_UC, getchr               ; wait for a keystroke
   calla cc_UC, sndchr               ; display keystroke on screen
   calla cc_UC, send_data            ; send keystroke to the CAN bus
   jmpr cc_UC, listen_port           ; loop
   


initialize_port:

   bset DP2.4                        ; configure P2.4 as output
   bset DP2.7                        ; configure P2.7 as output
   bset P2.4
   bset P2.7
   ret



initialize_can:

   bset IEN           ; enable interrupts

   mov r0, #48h       ; set interrupt control register
   mov XPOIC, r0      ; XPOIC <- 0100 1000

                      ; configure control/status register
   mov r0, #0046h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing register, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

                      ; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0,
                      ; Baud rate = 500 kb/s

                      ; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked according to this configuration
                                          
   mov r0, #5555h     ; initialize message objects 3-15
   mov MSG_CTL_3, r0  ; this application uses only the message  
   mov MSG_CTL_4, r0  ; objects 1 and 2. Message objects 2-15 are
   mov MSG_CTL_5, r0  ; declared invalid by resetting MSGVAL segment 
   mov MSG_CTL_6, r0  ; of the corresponding message control 
   mov MSG_CTL_7, r0  ; register.
   mov MSG_CTL_8, r0
   mov MSG_CTL_9, r0
   mov MSG_CTL_10, r0
   mov MSG_CTL_11, r0
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0
                          ; initialize message object 1
   mov r0, #5599h         ; MSG_CTL_1 <- 0101 0101 1001 1001 
   mov MSG_CTL_1, r0      ; message object 1 is valid. RXIE=1

   mov r0, #0000h         ; set arbitration registers for message 
   mov MSG_CTL_1 + 2, r0  ; object 1.
   mov r0, #0080h         ; message object 1 accepts messages with 
   mov MSG_CTL_1 + 4, r0  ; the identifier 1.

                          ; set message configuration register
   mov r0, #14h           ; for message object 1
   mov MSG_CTL_1 + 6, r0  ; DLC (data length)=1 bytes,
                          ; DIR=0 (receive), XTD=1 (extended)

                          ; initialize message object 2
   mov r0, #5595h         ; MSG_CTL_2 <- 0101 0101 1001 0101 
   mov MSG_CTL_2, r0      ; message object 2 is valid.
                   
                          ; set arbitration registers for message
   mov r0, #0000h         ; object 2.
   mov MSG_CTL_2 + 2, r0  ; message object 2 accepts messages with 
   mov MSG_CTL_2 + 4, r0  ; the identifier 0 (all 29 bits are 0).

                          ; set message configuration register 
   mov r0, #1Ch           ; for message object 2
   mov MSG_CTL_2 + 6, r0  ; DLC (data length)=1 byte,
                          ; DIR=1 (transmit), XTD=1 (extended)

                          ; initialization of the CAN controller 
   mov r0, #0006h         ; is completed. 
   mov CTL_ST, r0         ; clearing bit 6 (CCE) inhibits further 
                          ; access to the bit timing register, 
                          ; clearing bit 0(INIT) completes the 
   ret                    ; initialization.




send_data:
                             ; prepare transmit message object (2)
   mov r1, #0EAFFh           ; MSG_CTL_2 <- 1110 1010 1111 1111 
   mov MSG_CTL_2, r1         ; TXRQ=1, NEWDAT=1, CPUUPD=1

   movb MSG_CTL_2+7h, rl0    ; place data in the message object

                             ; transmit
   mov r1, #0F7FFh           ; MSG_CTL_2 <- 1111 0111 1111 1111
   mov MSG_CTL_2, r1         ; CPUUPD=0

   movb rl0, #4              ; signal transmission of message
   calla cc_UC, blink        ; by blinking LED in P2.4    
   ret




msg_isr:                     ; interrupt service routine

   movb rl0, #7              ; signal reception of message
   calla cc_UC, blink        ; by blinking LED in P2.7    

   movb r0, INT_ID           ; find out the interrupt source
   cmp r0, #3
   jmpr cc_NZ, interrupt_2 
   calla cc_UC, send_host    ; send ascii data to host
   mov r0, #0FFFDh           ; clear the INTPND of message object 1
   mov MSG_CTL_1, r0
interrupt_2:                 ; do nothing
   reti




send_host:

   movb rl0, MSG_CTL_1 + 7     ; get the ascii character
   calla cc_UC, sndchr         ; display on the screen   
   ret





#include "r66util.inc"
