;--------------------------- D3_1.ASM ------------------------------
;
; This program demonstrates duplex (bi-directional) communication
; through the CAN bus where messages of three different types 
; (each with a different identifier) can be transmitted in either
; direction. In a sense, D3_1.ASM is an extension of D0_1.ASM 
; because of its ability to transmit and receive multiple messages. 
; In this setup each node is connected to a host computer. Nodes 
; communicate with each other via the CAN bus. The other node should
; run D3_2.ASM. 
;
; As in D0_1.ASM, input buttons initiate message transfer. But in
; this case each CAN unit has three input buttons tied to P2.0, P2.2
; and P2.4. Pressing any of the buttons causes the corresponding
; input port to go low. The program constantly monitors the status
; of its input ports (P2.0, P2.2 and P2.4) and as soon as one of the
; ports goes low it generates a message and sends it over the CAN
; bus. 
;
; D3_1.ASM uses six message objects for data transfer; three receive
; objects and three transmit objects. The transmit object to be used
; is determined by the input button pressed. For example, button 1
; activates transmit object 1 (object #4) and button 2 activates
; transmit object 2 (object #5) and so on...
;
; Transmit objects are configured to carry one byte of data in their
; data fields. For testing purposes, each transmit object contains
; a different data byte. 
;  Transmit object  ----   1     2     3          
;  Data byte  ----------  'a'   'b'   'c'
;
; The program also checks its receive objects regularly to see if a 
; message has been received. If so, it strips the data off of the
; message and sends it to its host computer to be displayed on the
; screen.
;-------------------------------------------------------------------




; constant declarations
CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15
INPUT_1      equ P2.0
INPUT_2      equ P2.2
INPUT_3      equ P2.4




   org 04000h

   calla cc_UC, initialize_port
   calla cc_UC, initialize_can

start:
   calla cc_UC, check_ports    ; check if an input button is pressed 
   cmpb rl0, #0
   jmpa cc_Z, no_input         ; if not monitor the CAN bus       
   cmpb rl0, #1                ; if button1 is pressed
   calla cc_Z, send_msg_1      ; send message #1
   cmpb rl0, #2                ; if button2 is pressed
   calla cc_Z, send_msg_2      ; send message #2
   cmpb rl0, #3                ; if button 3 is pressed
   calla cc_Z, send_msg_3      ; send message #3
no_input:                      
   calla cc_UC, receive_msg    ; monitor the CAN bus to see if
   jmpa cc_UC, start           ; there is a CAN message
   



initialize_port:

   bclr DP2.0                  ; configure P2.0 as input
   bclr DP2.2                  ; configure P2.2 as input
   bclr DP2.4                  ; configure P2.4 as input   
   ret



initialize_can:
                      ; configure control_status register
   mov r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing reister, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

                      ; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                      ; Baud rate = 500 kb/s

                      ; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked in this configuration
                                          
   mov r0, #5555h     ; initialize message objects 7-15  
   mov MSG_CTL_7, r0  ; this application uses message objects 1 
   mov MSG_CTL_8, r0  ; through 6. Message objects 7-15 are
   mov MSG_CTL_9, r0  ; declared invalid by resetting MSGVAL segment
   mov MSG_CTL_10, r0 ; of the corresponding message control
   mov MSG_CTL_11, r0 ; register.
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0
                          ; initialize message objects 1-6
   mov r0, #5595h         ; MSG_CTL_n <- 0101 0101 1001 0101 
   mov MSG_CTL_1, r0      ; message object 1 is valid.
   mov MSG_CTL_2, r0      ; message object 2 is valid.
   mov MSG_CTL_3, r0      ; message object 3 is valid.
   mov MSG_CTL_4, r0      ; message object 4 is valid.
   mov MSG_CTL_5, r0      ; message object 5 is valid.
   mov MSG_CTL_6, r0      ; message object 6 is valid.

; set arbitration registers for message object 1
   mov r0, #0000h         ; message object 1 accepts messages with 
   mov MSG_CTL_1 + 2, r0  ; the identifier 1.
   mov r0, #0080h
   mov MSG_CTL_1 + 4, r0

; set message configuration register for message object 1
   mov r0, #14h           ; DLC (data length)=1 bytes,
   mov MSG_CTL_1 + 6, r0  ; DIR=0 (receive), XTD=1 (extended)
 
; set arbitration registers for message object 2
   mov r0, #0000h         ; message object 2 accepts messages with 
   mov MSG_CTL_2 + 2, r0  ; the identifier 2.
   mov r0, #0100h
   mov MSG_CTL_2 + 4, r0

; set message configuration register for message object 2
   mov r0, #14h           ; DLC (data length)=1 byte,
   mov MSG_CTL_2 + 6, r0  ; DIR=0 (receive), XTD=1 (extended)
 
; set arbitration registers for message object 3
   mov r0, #0000h         ; message object 3 accepts messages with 
   mov MSG_CTL_3 + 2, r0  ; the identifier 3.
   mov r0, #0180h
   mov MSG_CTL_3 + 4, r0

; set message configuration register for message object 3
   mov r0, #14h           ; DLC (data length)=1 bytes,
   mov MSG_CTL_3 + 6, r0  ; DIR=0 (receive), XTD=1 (extended)

; set arbitration registers for message object 4
   mov r0, #0000h         ; message object 4 transmits messages with 
   mov MSG_CTL_4 + 2, r0  ; the identifier 4.
   mov r0, #0200h
   mov MSG_CTL_4 + 4, r0

; set message configuration register for message object 4
   mov r0, #1Ch           ; DLC (data length)=1 byte,
   mov MSG_CTL_4 + 6, r0  ; DIR=1 (transmit), XTD=1 (extended)

; set arbitration registers for message object 5
   mov r0, #0000h         ; message object 5 accepts messages with 
   mov MSG_CTL_5 + 2, r0  ; the identifier 5.
   mov r0, #0280h
   mov MSG_CTL_5 + 4, r0

; set message configuration register for message object 5
   mov r0, #1Ch           ; DLC (data length)=1 bytes,
   mov MSG_CTL_5 + 6, r0  ; DIR=1 (transmit), XTD=1 (extended)

; set arbitration registers for message object 6
   mov r0, #0000h         ; message object 6 transmits messages with 
   mov MSG_CTL_6 + 2, r0  ; the identifier 6.
   mov r0, #0300h
   mov MSG_CTL_6 + 4, r0

; set message configuration register for message object 6
   mov r0, #1Ch           ; DLC (data length)=1 byte,
   mov MSG_CTL_6 + 6, r0  ; DIR=1 (transmit), XTD=1 (extended)

; initialization of the CAN controller is completed
   mov r0, #0000h     ; clearing bit 6 (CCE) inhibits further access 
   mov CTL_ST, r0     ; to the bit timing register, clearing bit 0
                      ; (INIT) completes the initialization
   ret 




check_ports:

   movb rl0, #0          ; reset input flag
   jb INPUT_1, check_2   ; if input1 is low (button1 is pressed)
   movb rl0, #1          ; input_flag=1
   ret
check_2:
   jb INPUT_2, check_3   ; if input2 is low (button2 is pressed)
   movb rl0, #2          ; input_flag=2
   ret
check_3:
   jb INPUT_3, nothing   ; if input3 is low (button3 is pressed)
   movb rl0, #3          ; input_flag=3
nothing:
   ret   




send_msg_1:
                              ; prepare transmit message object 
   mov r1, #0EAFFh            ; MSG_CTL_4 <- 1110 1010 1111 1111 
   mov MSG_CTL_4, r1          ; TXRQ=1, NEWDAT=1, CPUUPD=1

   movb rl1, #'a'
   movb MSG_CTL_4 + 7h, rl1   ; place data in the message object

   mov r1, #0F7FFh            ; transmit
   mov MSG_CTL_4, r1          ; MSG_CTL_4 <- 1111 0111 1111 1111
                              ; CPUUPD=0   
   mov r1, #0FBFFh            ; set CPUUPD to prevent any further
   mov MSG_CTL_4, r1          ; transmissions

   ret




send_msg_2:
                              ; prepare transmit message object 
   mov r1, #0EAFFh            ; MSG_CTL_5 <- 1110 1010 1111 1111 
   mov MSG_CTL_5, r1          ; TXRQ=1, NEWDAT=1, CPUUPD=1

   movb rl1, #'b'
   movb MSG_CTL_5 + 7h, rl1   ; place data in the message object

   mov r1, #0F7FFh            ; transmit
   mov MSG_CTL_5, r1          ; MSG_CTL_5 <- 1111 0111 1111 1111
                              ; CPUUPD=0   
   mov r1, #0FBFFh            ; set CPUUPD to prevent any further
   mov MSG_CTL_5, r1          ; transmissions

   ret




send_msg_3:
                              ; prepare transmit message object 
   mov r1, #0EAFFh            ; MSG_CTL_6 <- 1110 1010 1111 1111 
   mov MSG_CTL_6, r1          ; TXRQ=1, NEWDAT=1, CPUUPD=1

   movb rl1, #'c'
   movb MSG_CTL_6 + 7h, rl1   ; place data in the message object

   mov r1, #0F7FFh            ; transmit
   mov MSG_CTL_6, r1          ; MSG_CTL_6 <- 1111 0111 1111 1111
                              ; CPUUPD=0   
   mov r1, #0FBFFh            ; set CPUUPD to prevent any further
   mov MSG_CTL_6, r1          ; transmissions

   ret




receive_msg:

   calla cc_UC, check_rcv_obj ; find out if a message is received
   cmpb rl0, #1               ; if it is a type 1 message
   calla cc_Z, receive_msg_1  ; read it
   cmpb rl0, #2               ; if it is a type 2 message
   calla cc_Z, receive_msg_2  ; read it
   cmpb rl0, #3               ; if it is a type 3 message
   calla cc_Z, receive_msg_3  ; read it
   ret




check_rcv_obj:

   movb rl0, #0
   calla cc_UC, check_rcv_obj_1       ; check for message of type 1
   calla cc_UC, check_rcv_obj_2       ; check for message of type 2
   calla cc_UC, check_rcv_obj_3       ; check for message of type 3
   ret




check_rcv_obj_1:

   push r1
   mov r1, MSG_CTL_1
   and r1, #0300h             ; if (NEWDAT=1)
   cmp r1, #0200h             ; a message of type 1 has been 
   jmpa cc_NZ, no_msg_1       ; received
   mov r0, #1
no_msg_1:
   pop r1
   ret



check_rcv_obj_2:

   push r1
   mov r1, MSG_CTL_2
   and r1, #0300h             ; if (NEWDAT=1)
   cmp r1, #0200h             ; a message of type 2 has been 
   jmpa cc_NZ, no_msg_2       ; received
   mov r0, #2
no_msg_2:
   pop r1
   ret




check_rcv_obj_3:

   push r1
   mov r1, MSG_CTL_3
   and r1, #0300h             ; if (NEWDAT=1)
   cmp r1, #0200h             ; a message of type 3 has been 
   jmpa cc_NZ, no_msg_3       ; received
   mov r0, #3
no_msg_3:
   pop r1
   ret




receive_msg_1:

   movb rl0, MSG_CTL_1 + 7    ; get the data
   mov r1, #0FDFFh            ; clear NEWDAT to prepare for the next
   mov MSG_CTL_1, r1          ; reception
   calla cc_UC, sndchr        ; display the received data   
   ret



receive_msg_2:

   movb rl0, MSG_CTL_2 + 7    ; get the data
   mov r1, #0FDFFh            ; clear NEWDAT to prepare for the next
   mov MSG_CTL_2, r1          ; reception
   calla cc_UC, sndchr        ; display the received data   
   ret



receive_msg_3:

   movb rl0, MSG_CTL_3 + 7    ; get the data
   mov r1, #0FDFFh            ; clear NEWDAT to prepare for the next
   mov MSG_CTL_3, r1          ; reception
   calla cc_UC, sndchr        ; display the received data   
   ret



#include "r66util.inc"
