;------------------------ D6REC.ASM --------------------------------
;
; D6REC.ASM together with D6TR.ASM implements a fragmentation
; protocol. If the data to be transmitted is longer than the
; data field of a CAN message, it has to be broken up to several
; fragments and transmitted piece by piece. This necessitates
; the development and use of a fragmentation protocol. The protocol
; employed in this program is a rather simple one which is intended
; to demonstrate the key aspects of the problem.  
;
; The transmitting node, that runs D6TR.ASM, has the responsibility
; of sending a long stream of data (longer than 7 bytes) that does
; not fit in one CAN message. D6TR.ASM fragments the data and
; transmits it one at a time. Each transmission is initiated by a
; remote request from the message receiving node which runs 
; D6REC.ASM. The message receiving node generates a remote request
; any time P2.0 goes low. 
;
; The transmitted message not only contains the data but also a data
; header which informs the receiving node about the status of the 
; data transmitted. The data header occupies the first data byte in 
; the corresponding message object. It consists of the following
; fields:
; bit 7 --> fragmentation bit   (1)-fragmented, (0)-not fragmented    
; bit 6 --> last fragment bit   (1)-last fragment   (0)-not                                  
; bits 3-5 --> fragment count  (total number of fragments)
; bits 0-2 --> source MAC ID   (ID of the transmitting node) 
; 
; It is the transmitting node's reponsibility to prepare the data
; header and send it along with the actual data.
; The receiving node continues to generate remote requests until it
; receives the last fragment (a message with the last fragment bit
; set). 
;
; Upon the reception of a message the receiving node stores
; the data in the data segment in the external memory and updates
; the data pointer which points to the end of the data segment.
; When the last fragment is received all the fragments of the data
; are put together and processed (displayed on the screen in our
; example).  
;-------------------------------------------------------------------




; constant declarations
MONITOR      equ 0C100h              ; monitor program
DATA_POINTER equ  2000h              ; end of data segment stored here
DATA_SEGMENT equ  2100h              ; start address of data segment
CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15
INPUT_BUTTON equ P2.0





   org 04000h

   mov r0, #DATA_SEGMENT         ; data pointer points to the
   mov DATA_POINTER, r0          ; beginning of the data segment

   calla cc_UC, initialize_can

not_pressed:
   jb INPUT_BUTTON, not_pressed
send:
   calla cc_UC, send_remote
   calla cc_UC, receive_msg
   calla cc_UC, store_data
   calla cc_UC, clean_up
   calla cc_UC, is_fragmented      ; check if the message is 
   cmp r0, #0                      ; fragmented
   jmpa cc_Z, not_or_last          ; if not fragmented process data       
   calla cc_UC, is_last_frag       ; if it is not the last fragment
   cmp r0, #0 
   jmpa cc_Z, send                 ; wait for the next fragment
not_or_last:
   calla cc_UC, process_data 
   jmpa cc_UC, MONITOR  
   




initialize_can:

; configure control_status register
   mov r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing register, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                      ; Baud rate = 500 kb/s

; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked according to this configuration
                    
; initialize message objects 2-15
   mov r0, #5555h     ; this application uses only the message  
   mov MSG_CTL_2, r0  ; object 1. Message objects 2-15 are declared
   mov MSG_CTL_3, r0  ; invalid by resetting MSGVAL segment of the
   mov MSG_CTL_4, r0  ; corresponding message control register.
   mov MSG_CTL_5, r0
   mov MSG_CTL_6, r0
   mov MSG_CTL_7, r0
   mov MSG_CTL_8, r0
   mov MSG_CTL_9, r0
   mov MSG_CTL_10, r0
   mov MSG_CTL_11, r0
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0

; initialize message object 1
   mov r0, #5595h         ; MSG_CTL_1 <- 0101 0101 1001 0101 
   mov MSG_CTL_1, r0      ; message object 1 is valid.

; set arbitration registers for message object 1
   mov r0, #0000h         ; message object 1 accepts messages with 
   mov MSG_CTL_1 + 2, r0  ; the identifier 0 (all 29 bits are 0).
   mov MSG_CTL_1 + 4, r0

; set message configuration register for message object 1
   mov r0, #34h           ; DLC (data length)=3 bytes,
   mov MSG_CTL_1 + 6, r0  ; DIR=0 (receive), XTD=1 (extended)
 
; initialization of the CAN controller is completed
   mov r0, #0000h     ; clearing bit 6 (CCE) inhibits further access 
   mov CTL_ST, r0     ; to the bit timing register, clearing bit 0
                      ; (INIT) completes the initialization
ret 




send_remote:
                       ; send remote request
   mov r0, #0E7FFh     ; MSG_CTL_1 <- 1110 0111 1111 1111 
   mov MSG_CTL_1, r0   ; TXRQ=1, CPUUPD=0
   ret



receive_msg:
 
   mov r0, MSG_CTL_1   ; check message control register of object 1
   and r0, #0300h      ; to see if NEWDAT=1  
   cmp r0, #0200h
   jmpa cc_NZ, receive_msg
   ret



clean_up:

   push r0
   mov r0, #0F9FFh
   mov MSG_CTL_1, r0   ; clear NEWDAT, set CPUUPD
   mov r0, #0000h      ; clear RXOK
   mov CTL_ST, r0
   pop r0
   ret



is_fragmented:
 
   mov r0, #0               
   mov r1, #MSG_CTL_1             ; get the first data byte
   movb rl0, [r1+#7]              ; of the receive message object
   andb rl0, #80h                 ; check the fragmentation bit
   shr r0, #7h                    ; if bit=1 fragmented
   ret



is_last_frag:

   mov r0, #0
   mov r1, #MSG_CTL_1             ; get the first data byte
   movb rl0, [r1+#7]              ; of the receive message object
   andb rl0, #40h                 ; check the last fragment bit
   shr r0, #6                     ; if bit=1 last fragment
   ret



process_data:

   push r0
   push r1
   calla cc_UC, crlf
   mov r1, #DATA_SEGMENT          ; point to the first byte of data
         
get_byte:
   movb rl0, [r1]                 ; move data to rl0
   calla cc_UC, prthex            ; display data on the screen
   calla cc_UC, crlf
   add r1, #1                     ; point to the next data byte             
   cmp r1, DATA_POINTER           ; if the end of the data segment
   jmpa cc_NZ, get_byte           ; is reached exit the loop
   pop r1
   pop r0
   ret



store_data:

   push r0
   push r1
   push r2
   mov r1, #MSG_CTL_1
   add r1, #8                   ; r3 points to the first data byte
   mov r2, DATA_POINTER         ; r2 points to the data segment  
   calla cc_UC, get_DLC         ; find out how many bytes of data
                                ; is transmitted
store_byte:
   cmpd1 r0, #0                 ; decrement loop index if there is
   jmpa cc_Z, store_done        ; more data
   movb [r2], [r1]              ; get the next data byte in message
   add r2, #1                   ; and store it in the memory
   add r1, #1
   jmpa cc_UC, store_byte       ; repeat
store_done:
   mov DATA_POINTER, r2         ; mark where the data segment ends
   pop r2
   pop r1
   pop r0
   ret                        



get_DLC:

   mov r0, MSG_CTL_1 + 6        ; isolate the DLC segment of the
   and r0, #00F0h               ; configuration register for message
   shr r0, #4                   ; object 1 and shift right by four.
   sub r0, #1                   ; subtract 1 because one byte is 
   ret                          ; allocated for data header.




#include "r66util.inc"

