;--------------------------- D6TR.ASM ------------------------------
;
; D6REC.ASM together with D6TR.ASM implements a fragmentation
; protocol. If the data to be transmitted is longer than the
; data field of a CAN message, it has to be broken up to several
; fragments and transmitted piece by piece. This necessitates
; the development and use of a fragmentation protocol. The protocol
; employed in this program is a rather simple one which is intended
; to demonstrate the key aspects of the problem.  
;
; The transmitting node, that runs D6TR.ASM, has the responsibility
; of sending a long stream of data (longer than 7 bytes) that does
; not fit in one CAN message. D6TR.ASM fragments the data and
; transmits it one at a time. Each transmission is initiated by a
; remote request from the message receiving node which runs 
; D6REC.ASM. The message receiving node generates a remote request
; any time P2.0 goes low. 
;
; The transmitted message not only contains the data but also a data
; header which informs the receiving node about the status of the 
; data transmitted. The data header occupies the first data byte in 
; the corresponding message object. It consists of the following
; fields:
; bit 7 --> fragmentation bit   (1)-fragmented, (0)-not fragmented    
; bit 6 --> last fragment bit   (1)-last fragment   (0)-not                                  
; bits 3-5 --> fragment count  (total number of fragments)
; bits 0-2 --> source MAC ID   (ID of the transmitting node) 
; 
; It is the transmitting node's reponsibility to prepare the data
; header and send it along with the actual data.
; The receiving node continues to generate remote requests until it
; receives the last fragment (a message with the last fragment bit
; set). 
;
; Upon the reception of a message, the receiving node stores
; the data in the data segment in the external memory and updates
; the data pointer which points to the end of the data segment.
; When the last fragment is received all the fragments of the data
; are put together and processed (displayed on the screen in our
; example).  
;-------------------------------------------------------------------




; constant declarations
MDL          equ 0FE0Eh
MDH          equ 0FE0Ch
MONITOR      equ 0C100h              ; monitor program
DATA_POINTER equ  2000h              ; points to data in the memory
DATA_SEGMENT equ  2100h              ; start address of data segment
DATA_LEFT    equ  2002h              ; total length of transmitted data
FRAG_NO      equ  2004h              ; the fragment no of the last message
CTL_ST       equ 0EF00h              ; control/status register
BIT_TIM      equ 0EF04h              ; bit timing register
UP_GLM_L     equ 0EF08h              ; upper global mask long
LO_GLM_L     equ 0EFA0h              ; lower global mask long
MSG_CTL_1    equ 0EF10h              ; message control register 1
MSG_CTL_2    equ 0EF20h              ; message control register 2
MSG_CTL_3    equ 0EF30h              ; message control register 3
MSG_CTL_4    equ 0EF40h              ; message control register 4
MSG_CTL_5    equ 0EF50h              ; message control register 5
MSG_CTL_6    equ 0EF60h              ; message control register 6   
MSG_CTL_7    equ 0EF70h              ; message control register 7
MSG_CTL_8    equ 0EF80h              ; message control register 8
MSG_CTL_9    equ 0EF90h              ; message control register 9
MSG_CTL_10   equ 0EFA0h              ; message control register 10
MSG_CTL_11   equ 0EFB0h              ; message control register 11
MSG_CTL_12   equ 0EFC0h              ; message control register 12
MSG_CTL_13   equ 0EFD0h              ; message control register 13
MSG_CTL_14   equ 0EFE0h              ; message control register 14
MSG_CTL_15   equ 0EFF0h              ; message control register 15
REC_SIGNAL   equ P2.7
DATA_FIELD   equ 2
DATA_LENGTH  equ 0Dh



   org 04000h

   calla cc_UC, initialize_can
   calla cc_UC, initialize_data  
wait_rmt_rqst:     
   calla cc_UC, receiv_rmt_rqst
   calla cc_UC, send_data
   jmpr cc_UC, wait_rmt_rqst
   




initialize_can:
                      ; configure control_status register
   mov r0, #0041h     ; CTL_ST <- 0000 0000 0100 0001
   mov CTL_ST, r0     ; setting bit 6 (CCE) allows CPU access to the
                      ; bit timing register, setting bit 0 (INIT)
                      ; starts the initialization of CAN controller

                      ; configure bit timing register
   mov r0, #2F00h     ; BIT_TIM <- 0010 1111 0000 0000
   mov BIT_TIM, r0    ; TSEG1=15, TSEG2=2, SJW=0, BRP=0
                      ; Baud rate = 500 kb/s

                      ; set global mask registers                    
   mov r0, #0FFFFh    ; upper mask word -> 1111 1111 1111 1111
   mov UP_GLM_L, r0   ; lower mask word -> 1111 1000 1111 1111
   mov r0, #0F8FFh    ; all identifier bits of the incoming messages
   mov LO_GLM_L, r0   ; are checked in this configuration.
                    
   mov r0, #5555h     ; initialize message objects 2-15
   mov MSG_CTL_2, r0  ; this application uses only the message  
   mov MSG_CTL_3, r0  ; object 1. Message objects 2-15 are declared
   mov MSG_CTL_4, r0  ; invalid by resetting MSGVAL segment of the
   mov MSG_CTL_5, r0  ; corresponding message control register.
   mov MSG_CTL_6, r0
   mov MSG_CTL_7, r0
   mov MSG_CTL_8, r0
   mov MSG_CTL_9, r0
   mov MSG_CTL_10, r0
   mov MSG_CTL_11, r0
   mov MSG_CTL_12, r0
   mov MSG_CTL_13, r0
   mov MSG_CTL_14, r0
   mov MSG_CTL_15, r0
                          ; initialize message object 1
   mov r0, #5995h         ; MSG_CTL_1 <- 0101 1001 1001 0101 
   mov MSG_CTL_1, r0      ; message object 1 is valid. CPUUPD=1

   mov r0, #0000h         ; set arbitration registers for message
   mov MSG_CTL_1 + 2, r0  ; object 1
   mov MSG_CTL_1 + 4, r0  ; message object 1 accepts messages with 
                          ; the identifier 0 (all 29 bits are 0).

                          ; set message configuration register for 
   mov r0, #3Ch           ; message object 1
   mov MSG_CTL_1 + 6, r0  ; DLC (data length)=3 bytes, 
                          ; DIR=1 (transmit), XTD=1 (extended)
 
                          ; initialization of the CAN controller 
   mov r0, #0000h         ; is completed. Clearing bit 6 (CCE) 
   mov CTL_ST, r0         ; inhibits further access to the   
                          ; bit timing register, clearing bit 0
   ret                    ; (INIT) completes the initialization




initialize_data:

   mov r0, #DATA_SEGMENT    ; initially data pointer points to the
   mov DATA_POINTER, r0     ; beginning of data segment
   movb rl0, #10h           ; first data byte to be sent is 10h 
   mov r1, #DATA_SEGMENT
   mov r2, #0
next_data:
   cmpi1 r2, #DATA_LENGTH   ; a total of #DATA_LENGTH bytes of data
   jmpa cc_Z, done          ; will be transmitted
   movb [r1], rl0           ; copy the data byte to the data segment
   addb rl0, #1             ; next data byte
   add r1, #1               ; next address in the data segment
   jmpr cc_UC, next_data    ; repeat
done:
   movb rl0, #DATA_LENGTH   ; DATA_LEFT keeps track of how many more
   movb DATA_LEFT, rl0      ; bytes of data is to be transmitted 
   movb rl0, #0             
   movb FRAG_NO, rl0        ; reset the fragment number to zero.
   ret




send_data:
 
   push r0
   mov r0, #0FEFFh              ; set NEWDAT
   mov MSG_CTL_1, r0
   calla cc_UC, update_frag_no  ; increment fragmment no
   calla cc_UC, confg_header    ; prepare the data to be transmitted  
   calla cc_UC, write_data   
   calla cc_UC, transmit_msg    ; transmit message
   pop r0
   ret




receiv_rmt_rqst:

   push r0
rmt_rqst_loop:
   mov r0, MSG_CTL_1            ; check message control register 
   and r0, #0F000h              ; of object 1 to see if TXRQ=1 and
   cmp r0, #0A000h              ; RMTPND=1 (remote request received)    
   jmpa cc_NZ, rmt_rqst_loop    ; if so return
   pop r0
   ret



; ----------------------------------------------------------------
; returns the data header in r0 which contains the following:
; fragmentation bit -> bit 7
; last fragment bit -> bit 6  
; fragment count    -> bits 3-5
; source MAC ID     -> bits 0-2
; ----------------------------------------------------------------
confg_header:
  
   mov r0, #0 
   calla cc_UC, frag_bit       ; if fragmented, set fragmentation bit
   calla cc_UC, last_frag_bit  ; if last fragment, set last fragment bit
   calla cc_UC, frag_count     ; place the fragment count in header
   calla cc_UC, MAC_ID         ; place the MAC ID in header
   movb MSG_CTL_1+7, rl0       ; place data header
   ret





write_data:

   mov r0, DATA_POINTER
   mov r1, #MSG_CTL_1
   add r1, #8
   mov r2, #1
write_loop:
   movb [r1], [r0+]
   add r1, #1
   cmpi1 r2, #DATA_FIELD
   jmpa cc_NZ, write_loop
   mov DATA_POINTER, r0
   ret



transmit_msg:

   push r0
   mov r0, #0F7FFh                ; clear CPUUPD
   mov MSG_CTL_1, r0
transmitting:
   mov r0, CTL_ST
   and r0, #0800h
   cmp r0, #0800h
   jmpa cc_NZ, transmitting
   mov r0, #0FBFFh
   mov MSG_CTL_1, r0              ; set CPUUPD
   pop r0
   ret



frag_bit:

   push r0
   calla cc_UC, get_data_len     ; if the data to be transmitted
   cmp r0, #DATA_FIELD           ; fits into the data field       
   jmpa cc_ULE, clr_frag_bit     ; clear the fragmentation_bit    
   pop r0
   or r0, #0080h                 ; if not, fragmentation_bit=1
   ret
clr_frag_bit:
   pop r0  
   and r0, #0FF7Fh               ; fragmentation_bit=0         
   ret



last_frag_bit:

   push r0
   calla cc_UC, is_last_frag
   cmp r0, #0
   jmpa cc_Z, not_last
   pop r0                         ; the fragment no of this message
   or r0, #0040h                  ; set the last fragment bit    
   ret
not_last:
   pop r0                         ; if not
   and r0, #0FFBFh                ; clear the last fragment bit
   ret




is_last_frag:

   push r1
   calla cc_UC, get_frag_count
   movb rl1, rl0
   calla cc_UC, get_frag_no
   cmpb rl0, rl1
   jmpa cc_NZ, nott               ; if fragment count is equal to 
   mov r0, #1                     ; the current fragment number
   pop r1              
   ret                            ; then last fragment
nott:
   mov r0, #0
   pop r1
   ret




frag_count:

   push r1
   push r0
   calla cc_UC, get_frag_count
   mov r1, r0                     ; fragmentation count occupies
   shl r1, #3                     ; bits 3-5
   pop r0
   add r0, r1                     ; place the fragmentation count
   pop r1                         ; in header
   ret




MAC_ID:

   push r1
   push r0
   calla cc_UC, get_MAC_ID
   mov r1, r0
   pop r0
   add r0, r1
   pop r1
   ret




get_data_len:
   
   movb rh0, #0
   movb rl0, DATA_LEFT
   ret




update_frag_no:

   push r0
   movb rl0, FRAG_NO
   addb rl0, #1
   movb FRAG_NO, rl0
   pop r0
   ret
   



get_frag_count:

   push r1
   calla cc_UC, get_data_len
   movb MDL, rl0
   mov r1, #DATA_FIELD
   div r1  
   cmp MDH, #0
   jmpa cc_NZ, remainder
   movb rl0, MDL
   pop r1
   ret
remainder:
   movb rl0, MDL
   addb rl0, #1
   pop r1
   ret




get_MAC_ID:

   mov r0, #0
   ret




get_frag_no:
   
   movb rh0, #0
   movb rl0, FRAG_NO
   ret




#include "r66util.inc"
